﻿using NUnit.Framework;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FluentAssertions;
using IsbnCheckSum.Resources;
using IsbnCheckSum.Validation;

namespace IsbnCheckSum.Tests.Domain.Validation
{
	public class IsbnCodeFormatValidatorTests
	{
		[TestFixture]
		public class IsValidMethod : IsbnValidatorMethodBase
		{
			[Test]
			public void PassesForValidCodes()
			{
				// Arrange - just check the valid boundaries
				string[] inputs = 
						{ 
							"1234567890",		// Ten digits should pass
							"12-345-678-90",	// Ten significant digits with hyphens should pass
							"1234567890123",	// Thirteen digits should pass
							"12-3456-7890-123"	// Thirteen significant digits should pass
						};

				foreach (var input in inputs)
				{
					this.ShouldPass(input);
				}
			}

			[Test]
			public void FailsForInvalidCodeLengths()
			{
				// Arrange - just check the valid boundaries
				string[] inputs = 
						{ 
							"123456789",		// Nine digits should fail
							"12345678901",		// Eleven digits should fail
							"123456789012",		// Twelve digits should fail
							"12345678901234"	// Fourteen digits should fail
						};

				foreach (var input in inputs)
				{
					this.ShouldFail(input, ValidationMessages.InvalidIsbnLength);
				}
			}

			[Test]
			public void FailsForNotExactlyThreeHyphens()
			{
				// Arrange
				string[] inputs = { "1-234-56789", "1-23456789" };

				foreach (var input in inputs)
				{
					this.ShouldFail(input, ValidationMessages.InvalidIsbnHyphenCount);
				}
			}

			[Test]
			public void FailsForCodeWithHyphenAsFirstCharacter()
			{
				// Arrange
				string input = "-1-234-56789";

				this.ShouldFail(input, ValidationMessages.DontStartIsbnWithHyphens);
			}

			[Test]
			public void ThrowsExceptionForCodeWithConsecutiveHyphens()
			{
				// Arrange
				string input = "1-234--56789";

				this.ShouldFail(input, ValidationMessages.DontLetIsbnHaveConsecutiveHyphens);
			}

			private void ShouldPass(string input)
			{
				// Act
				bool result = this.Validator.IsValid(input);
				var errors = this.Validator.GetValidationErrors(input);

				// Assert
				result.Should().BeTrue();
				errors.Should().BeEmpty();
			}

			private void ShouldFail(string input, string expectedMessage)
			{
				// Act
				bool result = this.Validator.IsValid(input);
				var errors = this.Validator.GetValidationErrors(input);

				// Assert
				result.Should().BeFalse();
				errors.Should().Contain(expectedMessage);
			}
		}

		// TODO: Tests to ensure that CodeContainsCheckSum is working

		public abstract class IsbnValidatorMethodBase
		{
			protected ICanValidate<string> Validator { get; set; }

			[SetUp]
			protected virtual void SetUp()
			{
				this.Validator = new IsbnCodeFormatValidator();
			}
		}
	}
}
